@@ -54,7 +54,7 @@ needed.
 
 */
 
-size_t list_range_check(QPDFObjectHandle h, int index)
+py::size_t list_range_check(QPDFObjectHandle h, int index)
 {
     if (!h.isArray())
         throw py::type_error("object is not an array");
@@ -62,7 +62,24 @@ size_t list_range_check(QPDFObjectHandle h, int index)
         index += h.getArrayNItems(); // Support negative indexing
     if (!(0 <= index && index < h.getArrayNItems()))
         throw py::index_error("index out of range");
-    return (size_t)index;
+    return static_cast<py::size_t>(index);
+}
+
+inline bool typecode_is_bool(QPDFObject::object_type_e typecode)
+{
+    return typecode == QPDFObject::object_type_e::ot_boolean;
+}
+
+inline bool typecode_is_int(QPDFObject::object_type_e typecode)
+{
+    return typecode == QPDFObject::object_type_e::ot_integer;
+}
+
+inline bool typecode_is_numeric(QPDFObject::object_type_e typecode)
+{
+    return typecode == QPDFObject::object_type_e::ot_integer ||
+           typecode == QPDFObject::object_type_e::ot_real ||
+           typecode == QPDFObject::object_type_e::ot_boolean;
 }
 
 bool objecthandle_equal(QPDFObjectHandle self, QPDFObjectHandle other)
@@ -71,7 +88,7 @@ bool objecthandle_equal(QPDFObjectHandle self, QPDFObjectHandle other)
 
     // Uninitialized objects are never equal
     if (!self.isInitialized() || !other.isInitialized())
-        return false;
+        return false; // LCOV_EXCL_LINE
 
     // Indirect objects (objid != 0) with the same obj-gen are equal and same owner
     // are equal (in fact, they are identical; they reference the same underlying
@@ -83,31 +100,31 @@ bool objecthandle_equal(QPDFObjectHandle self, QPDFObjectHandle other)
         return self.getObjGen() == other.getObjGen();
     }
 
-    // If 'self' is a numeric type, convert both to Decimal objects
-    // and compare them as such.
-    if (self.getTypeCode() == QPDFObject::object_type_e::ot_integer ||
-        self.getTypeCode() == QPDFObject::object_type_e::ot_real ||
-        self.getTypeCode() == QPDFObject::object_type_e::ot_boolean) {
-        try {
-            auto a              = decimal_from_pdfobject(self);
-            auto b              = decimal_from_pdfobject(other);
-            py::object pyresult = a.attr("__eq__")(b);
-            bool result         = pyresult.cast<bool>();
-            return result;
-        } catch (const py::type_error &) {
-            return false;
-        }
+    auto self_typecode  = self.getTypeCode();
+    auto other_typecode = other.getTypeCode();
+
+    if (typecode_is_bool(self_typecode) && typecode_is_bool(other_typecode)) {
+        return self.getBoolValue() == other.getBoolValue();
+    } else if (typecode_is_int(self_typecode) && typecode_is_int(other_typecode)) {
+        return self.getIntValue() == other.getIntValue();
+    } else if (typecode_is_numeric(self_typecode) &&
+               typecode_is_numeric(other_typecode)) {
+        // If 'self' and 'other' are different numeric types, convert both to
+        // Decimal objects and compare them as such.
+        auto a              = decimal_from_pdfobject(self);
+        auto b              = decimal_from_pdfobject(other);
+        py::object pyresult = a.attr("__eq__")(b);
+        bool result         = pyresult.cast<bool>();
+        return result;
     }
 
     // Apart from numeric types, disimilar types are never equal
-    if (self.getTypeCode() != other.getTypeCode())
+    if (self_typecode != other_typecode)
         return false;
 
-    switch (self.getTypeCode()) {
+    switch (self_typecode) {
     case QPDFObject::object_type_e::ot_null:
         return true; // Both must be null
-    case QPDFObject::object_type_e::ot_boolean:
-        return self.getBoolValue() == other.getBoolValue();
     case QPDFObject::object_type_e::ot_name:
         return self.getName() == other.getName();
     case QPDFObject::object_type_e::ot_operator:
@@ -129,6 +146,12 @@ bool objecthandle_equal(QPDFObjectHandle self, QPDFObjectHandle other)
         // recurses into this function
         return (self.getDictAsMap() == other.getDictAsMap());
     }
+    case QPDFObject::object_type_e::ot_boolean:
+    case QPDFObject::object_type_e::ot_integer:
+    case QPDFObject::object_type_e::ot_real:
+        // LCOV_EXCL_START
+        throw std::logic_error("should have eliminated numeric types by now");
+        // LCOV_EXCL_STOP
     default:
         break;
     }
@@ -150,6 +173,16 @@ bool object_has_key(QPDFObjectHandle h, std::string const &key)
     return dict.hasKey(key);
 }
 
+bool array_has_item(QPDFObjectHandle haystack, QPDFObjectHandle needle)
+{
+    if (!haystack.isArray())
+        throw std::logic_error("object is not an array");
+
+    auto vec    = haystack.getArrayAsVector();
+    auto result = std::find(std::begin(vec), std::end(vec), needle);
+    return (result != std::end(vec));
+}
+
 QPDFObjectHandle object_get_key(QPDFObjectHandle h, std::string const &key)
 {
     if (!h.isDictionary() && !h.isStream())
@@ -205,6 +238,26 @@ std::pair<int, int> object_get_objgen(QPDFObjectHandle h)
     return std::pair<int, int>(objgen.getObj(), objgen.getGen());
 }
 
+PointerHolder<Buffer> get_stream_data(
+    QPDFObjectHandle &h, qpdf_stream_decode_level_e decode_level)
+{
+    try {
+        PointerHolder<Buffer> buf = h.getStreamData(decode_level);
+        // py::bytes will make a copy of the buffer, so releasing is fine
+        return buf;
+    } catch (const QPDFExc &e) {
+        // Make a new exception that has the objgen info, since QPDF's
+        // will not
+        std::string msg = e.getMessageDetail();
+        str_replace(msg, "getStreamData", "read_bytes");
+        throw QPDFExc(e.getErrorCode(),
+            e.getFilename(),
+            std::string("object ") + h.getObjGen().unparse(),
+            e.getFilePosition(),
+            msg);
+    }
+}
+
 void init_object(py::module_ &m)
 {
     py::enum_<QPDFObject::object_type_e>(m, "ObjectType")
@@ -215,7 +268,7 @@ void init_object(py::module_ &m)
         .value("integer", QPDFObject::object_type_e::ot_integer)
         .value("real", QPDFObject::object_type_e::ot_real)
         .value("string", QPDFObject::object_type_e::ot_string)
-        .value("name", QPDFObject::object_type_e::ot_name)
+        .value("name_", QPDFObject::object_type_e::ot_name)
         .value("array", QPDFObject::object_type_e::ot_array)
         .value("dictionary", QPDFObject::object_type_e::ot_dictionary)
         .value("stream", QPDFObject::object_type_e::ot_stream)
@@ -232,8 +285,24 @@ void init_object(py::module_ &m)
                 {sizeof(unsigned char)});
         });
 
-    py::bind_vector<std::vector<QPDFObjectHandle>>(m, "_ObjectList");
-    py::bind_map<std::map<std::string, QPDFObjectHandle>>(m, "_ObjectMapping");
+    py::bind_vector<ObjectList>(m, "_ObjectList") // Autoformat fix
+        .def("__repr__", [](ObjectList &ol) {
+            std::ostringstream ss;
+            bool first = true;
+            ss << "pikepdf._qpdf._ObjectList([";
+            for (auto &h : ol) {
+                if (first) {
+                    first = false;
+                } else {
+                    ss << ", ";
+                }
+                ss << objecthandle_repr(h);
+            }
+            ss << "])";
+            return ss.str();
+        });
+
+    py::bind_map<ObjectMap>(m, "_ObjectMapping");
 
     py::class_<QPDFObjectHandle>(m, "Object")
         .def_property_readonly("_type_code", &QPDFObjectHandle::getTypeCode)
@@ -357,12 +426,19 @@ void init_object(py::module_ &m)
             py::is_operator())
         .def("__copy__", [](QPDFObjectHandle &h) { return h.shallowCopy(); })
         .def("__len__",
-            [](QPDFObjectHandle &h) {
-                if (h.isDictionary())
-                    return (Py_ssize_t)h.getDictAsMap()
-                        .size(); // getKeys constructs a new object, so this is better
-                else if (h.isArray())
-                    return (Py_ssize_t)h.getArrayNItems();
+            [](QPDFObjectHandle &h) -> py::size_t {
+                if (h.isDictionary()) {
+                    // getKeys constructs a new object, so this is better
+                    return static_cast<py::size_t>(h.getDictAsMap().size());
+                } else if (h.isArray()) {
+                    int nitems = h.getArrayNItems();
+                    // LCOV_EXCL_START
+                    if (nitems < 0) {
+                        throw std::logic_error("Array items < 0");
+                    }
+                    // LCOV_EXCL_STOP
+                    return static_cast<py::size_t>(nitems);
+                }
                 if (h.isStream())
                     throw py::type_error(
                         "length not defined for object - "
@@ -518,14 +594,29 @@ void init_object(py::module_ &m)
             "keys.")
         .def("__contains__",
             [](QPDFObjectHandle &h, QPDFObjectHandle &key) {
+                if (h.isArray()) {
+                    return array_has_item(h, key);
+                }
                 if (!key.isName())
                     throw py::type_error("Dictionaries can only contain Names");
                 return object_has_key(h, key.getName());
             })
         .def("__contains__",
             [](QPDFObjectHandle &h, std::string const &key) {
+                if (h.isArray()) {
+                    throw py::type_error(
+                        "Testing `str in pikepdf.Array` is not supported due to "
+                        "ambiguity. Use `pikepdf.String('...') in pikepdf.Array.");
+                }
                 return object_has_key(h, key);
             })
+        .def("__contains__",
+            [](QPDFObjectHandle &h, py::object key) {
+                if (h.isArray()) {
+                    return array_has_item(h, objecthandle_encode(key));
+                }
+                return false;
+            })
         .def("as_list", &QPDFObjectHandle::getArrayAsVector)
         .def("as_dict", &QPDFObjectHandle::getDictAsMap)
         .def(
@@ -584,23 +675,23 @@ void init_object(py::module_ &m)
             })
         .def("__getitem__",
             [](QPDFObjectHandle &h, int index) {
-                size_t u_index = list_range_check(h, index);
+                auto u_index = list_range_check(h, index);
                 return h.getArrayItem(u_index);
             })
         .def("__setitem__",
             [](QPDFObjectHandle &h, int index, QPDFObjectHandle &value) {
-                size_t u_index = list_range_check(h, index);
+                auto u_index = list_range_check(h, index);
                 h.setArrayItem(u_index, value);
             })
         .def("__setitem__",
             [](QPDFObjectHandle &h, int index, py::object pyvalue) {
-                size_t u_index = list_range_check(h, index);
-                auto value     = objecthandle_encode(pyvalue);
+                auto u_index = list_range_check(h, index);
+                auto value   = objecthandle_encode(pyvalue);
                 h.setArrayItem(u_index, value);
             })
         .def("__delitem__",
             [](QPDFObjectHandle &h, int index) {
-                size_t u_index = list_range_check(h, index);
+                auto u_index = list_range_check(h, index);
                 h.eraseItem(u_index);
             })
         .def(
@@ -628,7 +719,7 @@ void init_object(py::module_ &m)
         .def(
             "get_stream_buffer",
             [](QPDFObjectHandle &h, qpdf_stream_decode_level_e decode_level) {
-                PointerHolder<Buffer> phbuf = h.getStreamData(decode_level);
+                auto phbuf = get_stream_data(h, decode_level);
                 return phbuf;
             },
             "Return a buffer protocol buffer describing the decoded stream.",
@@ -643,8 +734,7 @@ void init_object(py::module_ &m)
         .def(
             "read_bytes",
             [](QPDFObjectHandle &h, qpdf_stream_decode_level_e decode_level) {
-                PointerHolder<Buffer> buf = h.getStreamData(decode_level);
-                // py::bytes will make a copy of the buffer, so releasing is fine
+                auto buf = get_stream_data(h, decode_level);
                 return py::bytes((const char *)buf->getBuffer(), buf->getSize());
             },
             "Decode and read the content stream associated with this object.",
@@ -674,48 +764,8 @@ void init_object(py::module_ &m)
             py::arg("data"),
             py::arg("filter"),
             py::arg("decode_parms"))
-        .def_property_readonly("images",
-            [](QPDFObjectHandle &h) {
-                if (!h.isPageObject())
-                    throw py::type_error("Not a Page");
-                return h.getPageImages();
-            })
         .def("_inline_image_raw_bytes",
             [](QPDFObjectHandle &h) { return py::bytes(h.getInlineImageValue()); })
-        .def(
-            "page_contents_add",
-            [](QPDFObjectHandle &h, QPDFObjectHandle &contents, bool prepend) {
-                deprecation_warning(
-                    "pikepdf.Object.page_contents_add is deprecated; use "
-                    "pikepdf.Page.contents_add instead");
-                if (!h.isPageObject())
-                    throw py::type_error("Not a Page");
-
-                h.addPageContents(contents, prepend);
-            },
-            R"~~~(
-            Append or prepend to an existing page's content stream.
-
-            .. deprecated:: 2.14
-                Use :meth:`pikepdf.Page.contents_add` instead.
-            )~~~",
-            py::arg("contents"),
-            py::arg("prepend") = false,
-            py::keep_alive<1, 2>())
-        .def(
-            "page_contents_coalesce",
-            [](QPDFObjectHandle &h) {
-                deprecation_warning(
-                    "pikepdf.Object.page_contents_coalesce is deprecated; use "
-                    "pikepdf.Page.contents_coalesce instead");
-                return h.coalesceContentStreams();
-            },
-            R"~~~(
-            Coalesce an array of page content streams into a single content stream.
-
-            .. deprecated:: 2.14
-                Use :meth:`pikepdf.Page.contents_coalesce` instead.
-            )~~~")
         .def_property_readonly("_objgen", &object_get_objgen)
         .def_property_readonly("objgen",
             &object_get_objgen,
@@ -760,8 +810,7 @@ void init_object(py::module_ &m)
                 OperandGrouper og(whitelist);
                 QPDFObjectHandle::parseContentStream(h, &og);
                 if (!og.getWarning().empty()) {
-                    auto warn = py::module_::import("warnings").attr("warn");
-                    warn(og.getWarning());
+                    python_warning(og.getWarning().c_str());
                 }
                 return og.getInstructions();
             })
@@ -773,8 +822,21 @@ void init_object(py::module_ &m)
                 return h.unparse();
             },
             py::arg("resolved") = false,
-            "Convert PDF objects into their binary representation, optionally "
-            "resolving indirect objects.")
+            R"~~~(
+            Convert PDF objects into their binary representation, optionally
+            resolving indirect objects.
+
+            If you want to unparse content streams, which are a collection of
+            objects that need special treatment, use
+            :func:`pikepdf.unparse_content_stream` instead.
+
+            Returns ``bytes()`` that can be used with :meth:`Object.parse`
+            to reconstruct the ``pikepdf.Object``. If reconstruction is not possible,
+            a relative object reference is returned, such as ``4 0 R``.
+
+            Args:
+                resolved: If True, deference indirect objects where possible.
+            )~~~")
         .def(
             "to_json",
             [](QPDFObjectHandle &h, bool dereference = false) -> py::bytes {
@@ -805,7 +867,7 @@ void init_object(py::module_ &m)
                     indirect object.
 
             Returns:
-                bytes: JSON bytestring of object. The object is UTF-8 encoded
+                JSON bytestring of object. The object is UTF-8 encoded
                 and may be decoded to a Python str that represents the binary
                 values ``\x00-\xFF`` as ``U+0000`` to ``U+00FF``; that is,
                 it may contain mojibake.
@@ -879,12 +941,38 @@ void init_object(py::module_ &m)
         py::arg("op"));
     m.def("_Null", &QPDFObjectHandle::newNull, "Construct a PDF Null object");
 
-    py::class_<QPDFObjectHandle::ParserCallbacks, PyParserCallbacks>(m, "StreamParser")
-        .def(py::init<>())
-        .def("handle_object",
+    py::class_<QPDFObjectHandle::ParserCallbacks, PyParserCallbacks>(m,
+        "StreamParser",
+        R"~~~(
+            A simple content stream parser, which must be subclassed to be used.
+
+            In practice, the performance of this class may be quite poor on long
+            content streams because it creates objects and involves multiple
+            function calls for every object in a content stream, some of which
+            may be only a single byte long.
+
+            Consider instead using :func:`pikepdf.parse_content_stream`.
+        )~~~")
+        .def(py::init<>(), "You must call ``super.__init__()`` in subclasses.")
+        .def(
+            "handle_object",
             [](QPDFObjectHandle::ParserCallbacks &parsercallbacks,
-                QPDFObjectHandle &h) { parsercallbacks.handleObject(h); })
-        .def("handle_eof", &QPDFObjectHandle::ParserCallbacks::handleEOF);
+                QPDFObjectHandle &h,
+                size_t offset,
+                size_t length) { parsercallbacks.handleObject(h, offset, length); },
+            R"~~~(
+                This is an abstract method that must be overloaded in a subclass.
+
+                This function will be called back once for each object that is
+                parsed in the content stream.
+            )~~~")
+        .def("handle_eof",
+            &QPDFObjectHandle::ParserCallbacks::handleEOF,
+            R"~~~(
+                This is an abstract method that may be overloaded in a subclass.
+
+                Called at the end of a content stream.
+            )~~~");
 
     m.def("_encode", [](py::handle handle) { return objecthandle_encode(handle); });
     m.def("unparse", [](py::object obj) -> py::bytes {
