@@ -139,6 +139,11 @@ std::shared_ptr<QPDF> open_pdf(py::object filename_or_stream,
         q->pushInheritedAttributesToPage();
     }
 
+    if (!password.empty() && !q->isEncrypted()) {
+        python_warning(
+            "A password was provided, but no password was needed to open this PDF.");
+    }
+
     return q;
 }
 
@@ -420,20 +425,23 @@ void save_pdf(QPDF &q,
     Pl_PythonOutput output_pipe(description.c_str(), stream);
     w.setOutputPipeline(&output_pipe);
 
-    if (encryption.is(py::bool_(true)) && !q.isEncrypted()) {
+    bool encryption_is_truthy = encryption.equal(py::bool_(true));
+
+    if (encryption_is_truthy && !q.isEncrypted()) {
         throw py::value_error(
             "can't perserve encryption parameters on a file with no encryption");
     }
 
-    if ((encryption.is(py::bool_(true)) || py::isinstance<py::dict>(encryption)) &&
+    if ((encryption_is_truthy || py::isinstance<py::dict>(encryption)) &&
         (normalize_content || !stream_decode_level.is_none())) {
-        throw py::value_error(
-            "cannot save with encryption and normalize_content or stream_decode_level");
+        throw py::value_error("cannot save with encryption and normalize_content "
+                              "or stream_decode_level");
     }
 
-    if (encryption.is(py::bool_(true))) {
+    if (encryption_is_truthy) {
         w.setPreserveEncryption(true); // Keep existing encryption
-    } else if (encryption.is_none() || encryption.is(py::bool_(false))) {
+    } else if (encryption.is_none() || encryption.equal(py::bool_(false))) {
+        // Note: encryption.equal(py::bool_(false)) != !encryption_is_truthy
         w.setPreserveEncryption(false); // Remove encryption
     } else {
         setup_encryption(w, encryption);
